import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter_snow_base/flutter_snow_base.dart';
import 'package:flutter_snow_base/src/widgets/error_snow/error_snow_widget.dart';

class ListViewResourceWidget<T> extends StatelessWidget {
  final bool useSliver;
  final Resource<List<T>> resource;
  final Widget loadingTile;
  final Widget Function(T data) tileMapper;
  final Widget emptyWidget;
  final Future<void> Function() refresh;
  final EdgeInsets padding;
  final List<Widget> topWidgets;
  final Widget Function(AppException e) errorWidget;
  final Axis scrollDirection;
  final bool reverse;
  final ScrollController controller;
  final bool primary;
  final ScrollPhysics physics;
  final bool shrinkWrap;
  final bool addAutomaticKeepAlives;
  final bool addRepaintBoundaries;
  final bool addSemanticIndexes;
  final double cacheExtent;
  final int semanticChildCount;
  final DragStartBehavior dragStartBehavior;
  final ScrollViewKeyboardDismissBehavior keyboardDismissBehavior;
  final String restorationId;
  final Clip clipBehavior;
  final double itemExtent;
  final bool reorderableList;
  final ReorderCallback onReorder;
  final int loadingTileQuantity;

  const ListViewResourceWidget({
    Key key,
    this.useSliver = false,
    @required this.resource,
    @required this.loadingTile,
    @required this.tileMapper,
    this.refresh,
    this.emptyWidget,
    this.padding,
    this.topWidgets,
    this.errorWidget,
    this.scrollDirection = Axis.vertical,
    this.reverse = false,
    this.controller,
    this.primary,
    this.physics,
    this.shrinkWrap = true,
    this.itemExtent,
    this.addAutomaticKeepAlives = true,
    this.addRepaintBoundaries = true,
    this.addSemanticIndexes = true,
    this.cacheExtent,
    this.semanticChildCount,
    this.dragStartBehavior = DragStartBehavior.start,
    this.keyboardDismissBehavior = ScrollViewKeyboardDismissBehavior.manual,
    this.restorationId,
    this.clipBehavior = Clip.hardEdge,
    this.loadingTileQuantity = 2,
  })  : reorderableList = false,
        onReorder = null,
        super(key: key);

  const ListViewResourceWidget.reorderable({
    Key key,
    @required this.resource,
    @required this.loadingTile,
    @required this.tileMapper,
    @required this.onReorder,
    this.refresh,
    this.emptyWidget,
    this.topWidgets,
    this.errorWidget,
    this.padding,
    this.reverse = false,
    this.controller,
    this.shrinkWrap = true,
    this.loadingTileQuantity = 2,
  })  : scrollDirection = Axis.vertical,
        useSliver = false,
        primary = null,
        physics = null,
        itemExtent = null,
        cacheExtent = null,
        semanticChildCount = null,
        dragStartBehavior = DragStartBehavior.start,
        keyboardDismissBehavior = ScrollViewKeyboardDismissBehavior.manual,
        restorationId = null,
        clipBehavior = Clip.hardEdge,
        addAutomaticKeepAlives = true,
        addRepaintBoundaries = true,
        addSemanticIndexes = true,
        reorderableList = onReorder != null,
        super(key: key);

  const ListViewResourceWidget.sliver({
    Key key,
    this.useSliver = true,
    @required this.resource,
    @required this.loadingTile,
    @required this.tileMapper,
    this.refresh,
    this.emptyWidget,
    this.topWidgets,
    this.errorWidget,
    this.addAutomaticKeepAlives = true,
    this.addRepaintBoundaries = true,
    this.addSemanticIndexes = true,
    this.loadingTileQuantity = 2,
  })  : padding = null,
        scrollDirection = null,
        reverse = null,
        controller = null,
        primary = null,
        physics = null,
        shrinkWrap = null,
        itemExtent = null,
        cacheExtent = null,
        semanticChildCount = null,
        dragStartBehavior = null,
        keyboardDismissBehavior = null,
        restorationId = null,
        clipBehavior = null,
        reorderableList = false,
        onReorder = null,
        super(key: key);

  @override
  Widget build(BuildContext context) {
    var child = useSliver
        ? SliverList(
            delegate: SliverChildListDelegate.fixed(
              _generateList(
                resource,
              ),
              addAutomaticKeepAlives: addAutomaticKeepAlives,
              addRepaintBoundaries: addRepaintBoundaries,
              addSemanticIndexes: addSemanticIndexes,
            ),
          )
        : reorderableList
            ? ReorderableListView(
                padding: padding,
                scrollDirection: scrollDirection,
                reverse: reverse,
                scrollController: controller,
                onReorder: onReorder,
                header: topWidgets.length == 1
                    ? topWidgets.first
                    : Column(
                        mainAxisSize: MainAxisSize.min,
                        children: topWidgets,
                      ),
                children: _generateList(
                  resource,
                ),
              )
            : ListView(
                padding: padding,
                scrollDirection: scrollDirection,
                reverse: reverse,
                controller: controller,
                primary: primary,
                physics: physics,
                shrinkWrap: shrinkWrap,
                itemExtent: itemExtent,
                addAutomaticKeepAlives: addAutomaticKeepAlives,
                addRepaintBoundaries: addRepaintBoundaries,
                addSemanticIndexes: addSemanticIndexes,
                cacheExtent: cacheExtent,
                semanticChildCount: semanticChildCount,
                dragStartBehavior: dragStartBehavior,
                keyboardDismissBehavior: keyboardDismissBehavior,
                restorationId: restorationId,
                clipBehavior: clipBehavior,
                children: _generateList(
                  resource,
                ),
              );
    if (refresh != null && !useSliver) {
      return RefreshIndicator(
        onRefresh: refresh,
        child: child,
      );
    }
    return child;
  }

  List<Widget> _generateList(
    Resource<List<T>> resource,
  ) {
    var listWidgets = <Widget>[];
    if (resource.data != null) {
      listWidgets = resource.data.map<Widget>(tileMapper).toList();
    }
    switch (resource.status) {
      case Status.loading:
        for (var i = 0; i < loadingTileQuantity; i++) {
          listWidgets.add(loadingTile);
        }
        break;
      case Status.success:
        if (emptyWidget != null &&
            (resource.data == null || resource.data.isEmpty)) {
          listWidgets.add(emptyWidget);
        }
        break;
      case Status.failed:
        listWidgets.add(
          errorWidget != null
              ? errorWidget(resource.error)
              : ErrorSnowWidget(
                  resource.error.message,
                  onTryAgain: refresh,
                  key: UniqueKey(),
                ),
        );
        break;
      default:
        break;
    }
    if (topWidgets != null && !reorderableList) {
      listWidgets.insertAll(0, topWidgets);
    }
    return listWidgets;
  }
}
